/*******************************************************************************
 *
 *  hal_rfspi.c
 *
 *  Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "msp430.h"
#include "hal_rfspi.h"

/***************************************************************************//**
 * @file       hal_rfspi.c
 * @addtogroup hal_rfspi
 * @{
 ******************************************************************************/

/**********************************************************************//**
* @brief  Initializes the ports and serial communication peripherals to
*         communicate with the radio.
*
* @param  none
*
* @return none
**************************************************************************/

void halRfSpiInit(void)
{
	/* Setup CS to be used as GPIO */
	RF_PORT_SEL &= ~RF_SPI_CS;
	RF_PORT_OUT |= RF_SPI_CS;
    RF_PORT_DIR |= RF_SPI_CS;
	
    // Set up pins used by peripheral unit
    RF_PORT_SEL |= RF_SPI_SIMO + RF_SPI_SOMI + RF_SPI_CLK;
    RF_PORT_DIR |= RF_SPI_CLK + RF_SPI_SIMO;
	RF_PORT_DIR &= ~RF_SPI_SOMI;
	
    RF_PORT_OUT |= RF_SPI_SOMI;                                   
    RF_PORT_SEL |= RF_SPI_SIMO + RF_SPI_SOMI + RF_SPI_CLK;

	ME2 |= USPIE1;                            // Enable USART1 SPI mode
	UCTL1 = SWRST;                            // Disable USART state machine
	UCTL1 |= CHAR + SYNC + MM;                // 8-bit SPI Master **SWRST**
	UTCTL1 |= /*CKPL +*/ SSEL1 + SSEL0 + STC;     // SMCLK, 3-pin mode
	U1BR0 = 0x002;                            // SPICLK = SMCLK/2 = 500Hz
	U1BR1 = 0x000;                            //
	U1MCTL = 0x000;                           //
	// Release for operation
    U1IE &= ~(UTXIE1 + URXIE1); 
    UCTL1 &= ~SWRST; 
    IFG1 &= ~URXIFG1;
}

/**********************************************************************//**
* @brief  Disables the serial communications peripheral and clears the GPIO
*         settings used to communicate with the radio.
*
* @param  none
*
* @return none
**************************************************************************/

void halRfSpiShutDown(void)
{
    UCTL1 |= UCSWRST;
    RF_PORT_SEL &= ~(RF_SPI_SIMO + RF_SPI_SOMI + RF_SPI_CLK);
    RF_PORT_OUT &= ~RF_SPI_CS;
}

/**********************************************************************//**
* @brief  Set the chip select pin high.
*
* @param  none
*
* @return none
**************************************************************************/

void halRfSpiCSHigh(void)
{
    RF_PORT_OUT |= RF_SPI_CS;               // Set CSn high
}

/**********************************************************************//**
* @brief  Set the chip select pin high.
*
* @param  none
*
* @return none
**************************************************************************/

void halRfSpiCSLow(void)
{
    RF_PORT_OUT &= ~RF_SPI_CS;               // Set CSn low
}

/***************************************************************************//**
 * @brief   Read a frame of bytes via SPI
 * @param   pBuffer Place to store the received bytes
 * @param   size Indicator of how many bytes to receive
 * @return  None
 ******************************************************************************/
void halRFSpiReadWriteFrame(uint8_t *pTxBuffer, uint8_t *pRxBuffer, uint16_t size)
{
    // Clock the actual data transfer and receive the bytes
    while (size--){
        while(!(IFG2&UTXIFG1));                     // Wait while not ready for TX
        U1TXBUF = *pTxBuffer++;                           // Write TX byte
        while(!(IFG2&URXIFG1));                    // Wait for RX buffer (full)
        *pRxBuffer++ = U1RXBUF;
    }
}
/***************************************************************************//**
 * @}
 ******************************************************************************/

