/*******************************************************************************
 *
 *  hal_usb.c
 *
 *  Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "msp430.h"
#include  <msp430xG46x.h>
#include "hal_uart.h"

#ifdef __CCS__
	unsigned char g_ucRxBuffer[UART_IF_BUFFER];
#else
	__no_init unsigned char g_ucRxBuffer[UART_IF_BUFFER];
#endif

volatile unsigned short g_usRxBufferCounter = 0;
volatile unsigned char uart_have_cmd =0;

/**********************************************************************//**
* @brief  Initializes the serial communications peripheral and GPIO ports
*         to communicate with the TUSB3410.
*
* @param  none
*
* @return none
**************************************************************************/

void halUartInit(void)
{
	g_usRxBufferCounter = 0;
	uart_have_cmd = 0;
	
    UART_PORT_SEL |= UART_PIN_RXD + UART_PIN_TXD;
    UART_PORT_DIR |= UART_PIN_TXD;
    UART_PORT_DIR &= ~UART_PIN_RXD;
	UCA0CTL1 |= UCSSEL_2;                     // SMCLK
	UCA0BR0 = 0x41;                           // 8MHz /115200 =72
	UCA0BR1 = 0x03;                         // 8MHz/9600 =833.3 = 0x341
	UCA0MCTL = 0x02;                          // Modulation
	UCA0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**
	IE2 |= UCA0RXIE;                          // Enable USCI_A0 RX interrupt
}

/**********************************************************************//**
* @brief  Disables the serial communications peripheral and clears the GPIO
*         settings used to communicate with the TUSB3410.
*
* @param  none
*
* @return none
**************************************************************************/

void halUartShutDown(void)
{
    IE2 &= ~UCA0RXIE;
    U1CTL = UCSWRST;                     //Reset State
    UART_PORT_SEL &= ~(UART_PIN_RXD + UART_PIN_TXD);
    UART_PORT_DIR |= UART_PIN_TXD;
    UART_PORT_DIR |= UART_PIN_RXD;
    UART_PORT_OUT &= ~(UART_PIN_TXD + UART_PIN_RXD);
}

/**********************************************************************//**
* @brief  Sends a character over UART to the TUSB3410.
*
* @param  character The character to be sent.
*
* @return none
**************************************************************************/

void halUartSendChar(char character)
{
    while(!(IFG2&UCA0TXIFG));
     UCA0TXBUF = character;
}

/**********************************************************************//**
* @brief  Sends a string of characters to the TUSB3410
*
* @param  string[] The array of characters to be transmit to the TUSB3410.
*
* @param  length   The length of the string.
*
* @return none
**************************************************************************/

void halUartSendString(char string[], unsigned char length)
{
    unsigned char i;

    for (i = 0; i < length; i++)
        halUartSendChar(string[i]);
}

/************************************************************************/
#pragma vector=USCIAB0RX_VECTOR
__interrupt void UART_ISR(void)
{
    g_ucRxBuffer[g_usRxBufferCounter] =  UCA0RXBUF;
	//if(g_ucRxBuffer[g_usRxBufferCounte] == 0x0D )
	uart_have_cmd = g_ucRxBuffer[1];
	if(++g_usRxBufferCounter >= UART_IF_BUFFER)
		g_usRxBufferCounter = 0;
	

}

